/*
 * Copyright (c) 2013-2016 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Misfired Triggers grid in the top-level Misfired Triggers panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.misfire.RefreshMisfiredTriggersGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    // TODO: implement
    if ( true )
      return;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.MisfiredTriggers &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId ) )
    {
      var misfiredTriggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getMisfiredTriggersTab(
          me.connectionId );

      var misfiredTriggersGrid = misfiredTriggersTab.down( '#misfiredTriggersGrid' );

      var misfiredTriggersGridStore = misfiredTriggersGrid.getStore();

      misfiredTriggersGridStore.getProxy().url =
          QDesk.util.constant.ActionUrl.quartz.misfireActionListForScheduler;

      var extraStoreParams = {
        connectionId : me.connectionId
      };

      // if misfired triggers index enabled, we add the filter query text param
      if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzMisfiredTriggersIndexEnabled(
              me.connectionId ) )
      {
        var filterQueryText = misfiredTriggersGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      misfiredTriggersGridStore.getProxy().extraParams = extraStoreParams;

      misfiredTriggersGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
