/*
 * Copyright (c) 2013-2016 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.store.scheduler.quartz.QuartzMisfiredTriggersStore',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.view.widget.WMaxTodayDateField',
    'QDesk.view.widget.WSearchField',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzMisfiredTriggersGrid',

  cls : 'misfiredTriggersGrid',

  scrollable : true,

  //border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    if ( !config.qdesk.maxFetchSize )
      Ext.Error.raise( 'Required config parameter missing: qdesk.maxFetchSize' );

    var misfiredTriggersStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzMisfiredTriggersStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'WMaxTodayDateField',
          itemId : 'maxFetchDateSelect',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.fieldLabelMaxFetchDateSelect' ),
          editable : false,
          format : QDesk.util.constant.DateFormat.dateFormat,
          value : new Date(),
          maxValue : new Date(),
          labelAlign : 'right',
          labelWidth : 55,
          width : 155,
          disabled : true,
          listeners : {
            change : {
              fn : function ( picker, newValue )
              {
                this.fireEvent( 'changeMaxFetchDate', newValue );
              }
            }
          }
        },
        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [ 10, '10' ],
              [ 20, '20' ],
              [ 50, '50' ],
              [ 100, '100' ],
              [ 500, '500' ],
              [ 1000, '1000' ]
              // [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 72,
          width : 172,
          editable : false,
          disabled : true,
          listeners : {
            change : {
              fn : function ( combo, newValue )
              {
                this.fireEvent( 'changeMaxFetchSize', newValue );
              }
            }
          }
        },
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextFieldFilterQueryText' ),
          store : misfiredTriggersStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ],


      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.6
        },
        {
          xtype : 'datecolumn',
          itemId : 'detectedAtColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textDetectedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipDetectedAtColumn' ),
          dataIndex : 'detectedAt',
          sortable : true,
          groupable : false,
          flex : 1.7
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerNameColumn' ),
          dataIndex : 'schedulerName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerInstanceIdColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerInstanceIdColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerInstanceIdColumn' ),
          dataIndex : 'schedulerInstanceId',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerObjectNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerVersionColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerVersionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerVersionColumn' ),
          dataIndex : 'schedulerVersion',
          renderer : function ( value )
          {
            return value.major + '.' + value.minor + '.' + value.maintenance;
          },
          sortable : true,
          groupable : false,
          flex : 1.1
        },
        /* End of scheduler related columns */

        /* Trigger related columns */
        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'triggerGroupName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'triggerName',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'templatecolumn',
          itemId : 'triggerTypeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerTypeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerTypeColumn' ),
          dataIndex : 'trigger.type',
          tpl : '{[Ext.isEmpty( values.triggerType ) ? QDesk.Msg.get("QDesk.unknown") : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( values.triggerType )]}',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        {
          itemId : 'triggerMisfireInstructionColumn',
          format : '0',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerMisfireInstructionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerMisfireInstructionColumn' ),
          dataIndex : 'triggerMisfireInstruction',
          sortable : false,
          groupable : false,
          /* Need custom renderer to display misfire instruction name */
          renderer : function ( value, metadata, record )
          {
            var triggerType = record.get( 'triggerType' );

            if ( triggerType )
            {
              var misfireInstruction =
                  QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionFromNumber( triggerType,
                      value );

              return QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionName( triggerType,
                  misfireInstruction );
            }
            else
            {
              // if we do not know the trigger type, then we are not able to determine the misfire instruction name;
              // so we display the numeric constant instead
              return QDesk.Msg.get( 'QDesk.unknown' ) + ' (' + value.toString() + ')';
            }
          },
          flex : 1
        },
        /* End of trigger related columns */

        {
          xtype : 'datecolumn',
          itemId : 'missedFireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textMissedFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipMissedFireTimeColumn' ),
          dataIndex : 'missedFireTime',
          sortable : true,
          groupable : false,
          flex : 1.7
        },

        /* Job related columns */
        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobGroupName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobName',
          sortable : true,
          groupable : false,
          flex : 1
        }
        /* End of job related columns */
      ],

      store : misfiredTriggersStore,

      qdesk : {
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );

    var maxFetchSize = QDesk.ApplicationConfig.get( me.qdesk.gridId + '.maxFetchSize' );
    me.down( '#maxFetchSizeSelect' ).setValue( maxFetchSize );
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();

    var extra = {};
    extra[ me.qdesk.gridId + '.maxFetchSize' ] = maxFetchSize;

    return extra;
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.getStore().on( 'load', function ( store, records, successful )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          var jobsTab = me.up( '#jobsTab' );
          var triggersTab = me.up( '#triggersTab' );

          // default empty text
          var emptyText = QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoMatchingData' );

          if ( jobsTab )
          {
            var selectedJobs = jobsTab.down( '#jobsGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedJobs ) )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoJobSelected' );
            }
            else
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoMatchingDataForJob' );
            }
          }
          else if ( triggersTab )
          {
            var selectedTriggers = triggersTab.down( '#triggersGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedTriggers ) )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoTriggerSelected' );
            }
            else
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoMatchingDataForTrigger' );
            }
          }

          view.emptyText = emptyText;

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  }
} );
